
//////////////////////////////////////////////////////////////
// La communication venant et en direction du master est geree par le hardware
// La communication venant et en direction des octaves suivant est generee par SoftwareSerial

// De maniere alternative on pourrait par le hardware , cela permettrait de laisser le hardware
// s'occuper de la partie intense en communications et le retour vers le master, plus simple, gere
// par le soft

// Tous les messages terminent par '\n'

// Reset général: recu du master: si le message commence par 'R', si c'est une erreur de comm tout
//                est reseté ex ... si le master envoit 'RRRRR' tout est sur d'etre remis à 0, reponse
//                par un "R"+numOctave+"RR\n" vers le master pour signaler
// Assignation du numero d'octave: recu du master: si le message commence par 'S', le numero qui
//                suit est le numero d'octave courant (n) et 'S'+(n+1)+'\n' est transmit et
//                'S'+n+'\n' est renvoye au master
// Demande de l'état des touches: recu du master : precede du numero d'octave, puis 'Q', reponse par
//                un message de la forme numOctave+'T'+etat des touches en hexadecimal dans un string
// Demande de d'allumage de touche: recu du master : precede du numero d'octave, puis 'O' suivi du
//                numero de la touche et du code de la couleur a afficher ('0' pas de couleur), reponse
//                par un message de la forme numOctave+'K'+numero de la touche+code de la couleur
//////////////////////////////////////////////////////////////

#include <Adafruit_PWMServoDriver.h>

#define HARD_SPEED 38400
#define SOFT_SPEED 38400



// called this way, it uses the default address 0x40
Adafruit_PWMServoDriver pwm0 = Adafruit_PWMServoDriver(0x40);
Adafruit_PWMServoDriver pwm1 = Adafruit_PWMServoDriver(0x41);
Adafruit_PWMServoDriver pwm2 = Adafruit_PWMServoDriver(0x42);

#include <SPI.h>

#include <Wire.h>
#include <MPR121.h>
#include <MPR121_defs.h>

#include <SoftwareSerial.h>

bool bFirst = true;
bool bInit = false;
byte colorKey[ 12 ] = {1, 2, 3, 4, 5, 6, 7, 1, 2, 3, 5, 4}; // std color for the key
byte touchStatus[ 12 ]; // 0 released - 1 pressed
byte touchColors[ 12 ];

byte tlcColor[ 8 ][ 3 ] =
{
    0, 0, 0,
    9, 0, 0,
    0, 9, 0,
    0, 0, 9,
    9, 9, 0,
    9, 0, 9,
    0, 9, 9,
    9, 9, 9
};

byte tlcIdx[ 12 ][ 3 ] = {
1, 2, 3,
4, 5, 6,
7, 8, 9,
10, 11, 12,
17, 18, 19,
20, 21, 22,
23, 24, 25,
26, 27, 28,
33, 34, 35,
36, 37, 38,
39, 40, 41,
42, 43, 44
};

#define toSlaveSerial Serial
SoftwareSerial toMasterSerial( 4, 5 ); // RX, TX

byte stateReturnPath; // length of messages
byte stateMaster;

char msgFromSlave[ 100 ];  // messages
char msgFromMaster[ 100 ];
char dbgMaster[ 100 ];

char octaveNumber = '-';

// Setup MPR
int touche[ 12 ] = { 0, 1, 0, 1, 0, 0, 1, 0, 1, 0, 1, 0 };
int toucheup[ 12 ] = { 0, 1, 0, 1, 0, 0, 1, 0, 1, 0, 1, 0 };
int touchedn[ 12 ] = { 0, 1, 0, 1, 0, 0, 1, 0, 1, 0, 1, 0 };
int up1 = 12;
int down1 = 11;
int up2 = 23;
int down2 = 22;

//////////////////////////////////////////////////////////////
void debug( char *txt )
{
// masterSerial.write( 'd' );
// masterSerial.write( octaveNumber + 0x30 );
// masterSerial.write( "<=", 2 );
// masterSerial.write( txt );
}

//////////////////////////////////////////////////////////////
void setup( )
{
toSlaveSerial.begin( SOFT_SPEED );
//while ( !slaveSerial );

toMasterSerial.begin( SOFT_SPEED );
//while ( !masterSerial );

bInit = false;
bFirst = true;
stateReturnPath = 0;
stateMaster = 0;

if ( !MPR121.begin( 0x5A ) )
    {
    debug( "error setting up MPR121\n" );
    switch ( MPR121.getError( ) )
        {
        case NO_ERROR:
            debug( "no error\n" );
            break;
        case ADDRESS_UNKNOWN:
            debug( "incorrect address\n" );
            break;
        case READBACK_FAIL:
            debug( "readback failure\n" );
            break;
        case OVERCURRENT_FLAG:
            debug( "overcurrent on REXT pin\n" );
            break;
        case OUT_OF_RANGE:
            debug( "electrode out of range\n" );
            break;
        case NOT_INITED:
            debug( "not initialised\n" );
            break;
        default:
            debug( "unknown error\n" );
            break;
        }
    while ( 1 );
    }

MPR121.setInterruptPin( 2 );
MPR121.setTouchThreshold( up2 ); // default 40 = touch
MPR121.setReleaseThreshold( down2 );  // default 20 = touch
for ( int i = 0; i < 12; i++ )
    {
    int up = up1;
    int down = down1;
    if ( touche[ i ] == 1 )
        {
        up = up2;
        down = down2;
        }
    MPR121.setTouchThreshold( i, down ); // default 40 = touch
    MPR121.setReleaseThreshold( i, up );  // default 20 = touch
    }

//init pca
  pwm0.begin();
  pwm0.setPWMFreq(1600);  // This is the maximum PWM frequency
  pwm1.begin();
  pwm1.setPWMFreq(1600);  // This is the maximum PWM frequency
  pwm2.begin();
  pwm2.setPWMFreq(1600);  // This is the maximum PWM frequency

}

//////////////////////////////////////////////////////////////
void reset( )
{
bInit = false;
stateReturnPath = 0;
stateMaster = 0;

toSlaveSerial.write( "RRRRR\n" );
}

//////////////////////////////////////////////////////////////
void askForReset( )
{
toMasterSerial.write( "uuuuu\n" );

stateReturnPath = 0;
stateMaster = 0;

}

//////////////////////////////////////////////////////////////
void notifyKeyChanged( char touchId, char state )
{
msgFromMaster[ 0 ] = 'n';
msgFromMaster[ 1 ] = octaveNumber + 0x30;
msgFromMaster[ 2 ] = touchId + 0x30;
msgFromMaster[ 3 ] = state + 0x30;
msgFromMaster[ 4 ] = '\n';
msgFromMaster[ 5 ] = 0;
toMasterSerial.write( msgFromMaster );

}

//////////////////////////////////////////////////////////////
void ledOnOff( char id, char color )
{
if ( id > 11 )
    {
    for ( id = 0; id < 12; id++ )
        {
        touchColors[ id ] = color;
        }
    }
else
    {
    touchColors[ id ] = color;
    }

tlcUpdate( );
}

//////////////////////////////////////////////////////////////
void messageFromSlave( char newIn )
{
if ( newIn == '$' ) newIn = '\n';
msgFromSlave[ stateReturnPath++ ] = newIn;
msgFromSlave[ stateReturnPath ] = 0;
if ( newIn == '\n' )
    { // fin de message
    toMasterSerial.write( msgFromSlave );
    stateReturnPath = 0;
    }
}

//////////////////////////////////////////////////////////////
void tlcUpdate( )
{
for ( int i = 0; i < 4; i++ )
    {
    pwm0.setPWM(i*3+2, 0, 455 * tlcColor[ touchColors[ i+0 ] ][ 0 ] );
    pwm0.setPWM(i*3+3, 0, 455 * tlcColor[ touchColors[ i+0 ] ][ 1 ] );
    pwm0.setPWM(i*3+4, 0, 455 * tlcColor[ touchColors[ i+0 ] ][ 2 ] );
    pwm1.setPWM(i*3+2, 0, 455 * tlcColor[ touchColors[ i+4 ] ][ 0 ] );
    pwm1.setPWM(i*3+3, 0, 455 * tlcColor[ touchColors[ i+4 ] ][ 1 ] );
    pwm1.setPWM(i*3+4, 0, 455 * tlcColor[ touchColors[ i+4 ] ][ 2 ] );
    pwm2.setPWM(i*3+2, 0, 455 * tlcColor[ touchColors[ i+8 ] ][ 0 ] );
    pwm2.setPWM(i*3+3, 0, 455 * tlcColor[ touchColors[ i+8 ] ][ 1 ] );
    pwm2.setPWM(i*3+4, 0, 455 * tlcColor[ touchColors[ i+8 ] ][ 2 ] );
    }
}

//////////////////////////////////////////////////////////////
void messageFromMaster( char newIn )
{
if ( newIn == '$' ) newIn = '\n';

msgFromMaster[ stateMaster++ ] = newIn;
msgFromMaster[ stateMaster ] = 0;

if ( newIn == '\n' )
    { // fin de message
//     if ( bInit == true )
//         {
//         toMasterSerial.write( "d" );
//         toMasterSerial.write( msgFromMaster );
//         }
//     else
//         {
//         toMasterSerial.write( "D" );
//         toMasterSerial.write( msgFromMaster );
//         }
    switch ( msgFromMaster[ 0 ] )
        {
        case 'A': // set touch & release threshold
            if ( bInit == true )
                {
                for ( int i = 0; i < 12; i++ )
                    {
                    int down = 10 * (msgFromMaster[ 1 ] - 0x30) + (msgFromMaster[ 2 ] - 0x30);
                    if ( touche[ i ] == 0 )
                        MPR121.setTouchThreshold( i, down ); // default 40 = touch
                    }
                MPR121.updateTouchData( );
                toSlaveSerial.write( msgFromMaster );
                }
            break;
        case 'B': // set touch & release threshold
            if ( bInit == true )
                {
                for ( int i = 0; i < 12; i++ )
                    {
                    int val = 10 * (msgFromMaster[ 1 ] - 0x30) + (msgFromMaster[ 2 ] - 0x30);
                    if ( touche[ i ] == 0 )
                        MPR121.setReleaseThreshold( i, val );  // default 20 = touch
                    }
                MPR121.updateTouchData( );
                toSlaveSerial.write( msgFromMaster );

                }
            break;
        case 'C': // set touch & release threshold
            if ( bInit == true )
                {
                for ( int i = 0; i < 12; i++ )
                    {
                    int down = 10 * (msgFromMaster[ 1 ] - 0x30) + (msgFromMaster[ 2 ] - 0x30);
                    if ( touche[ i ] == 1 )
                        MPR121.setTouchThreshold( i, down ); // default 40 = touch
                    }
                MPR121.updateTouchData( );
                toSlaveSerial.write( msgFromMaster );
                }
            break;
        case 'D': // set touch & release threshold
            if ( bInit == true )
                {
                for ( int i = 0; i < 12; i++ )
                    {
                    int val = 10 * (msgFromMaster[ 1 ] - 0x30) + (msgFromMaster[ 2 ] - 0x30);
                    if ( touche[ i ] == 1 )
                        MPR121.setReleaseThreshold( i, val );  // default 20 = touch
                    }
                MPR121.updateTouchData( );
                toSlaveSerial.write( msgFromMaster );
                }
            break;
        case 'O':
            if ( bInit == true )
                {
                if ( msgFromMaster[ 1 ] != octaveNumber + 0x30 )
                    {
                    toSlaveSerial.write( msgFromMaster, stateMaster ); //not talking to me: bypass
                    }
                else
                    {
                    ledOnOff( msgFromMaster[ 2 ] - 0x30, msgFromMaster[ 3 ] - 0x30 );
                    //msgMaster[0] = 'o';
                    //masterSerial.write( msgMaster );
                    }
                }
            break;
        case 'P':
            if ( bInit == true )
                {
                if ( msgFromMaster[ 1 ] != octaveNumber + 0x30 )
                    {
                    toSlaveSerial.write( msgFromMaster, stateMaster ); //not talking to me: bypass
                    }
                else
                    {
                    colorKey[ msgFromMaster[ 2 ] - 0x30 ] = msgFromMaster[ 3 ] - 0x30;
    /*				msgMaster[0] = 'p';
                    masterSerial.write( msgMaster );*/
                    }
                }
            break;
        case 'R': // Reset all
            reset( );
            break;
        case 'S': // set octave number
        {
        if ( bInit == true )
            {
            //toMasterSerial.write( "Sending to next" ); //not talking to me: bypass
            toSlaveSerial.write( msgFromMaster, stateMaster ); //not talking to me: bypass
            }
        else
            {
            octaveNumber = msgFromMaster[ 1 ] - 0x30;
            msgFromMaster[ 0 ] = 's'; 
            toMasterSerial.write( msgFromMaster, stateMaster ); //not talking to me: bypass
            bInit = true;
            }
        }
        break;
        case 'T': // bring back all touchstatus to Master
        {
        int j = 0;
        msgFromMaster[ j++ ] = 't';
        msgFromMaster[ j++ ] = octaveNumber + 0x30;
        for ( int i = 0; i < 12; i++ )
            {
            msgFromMaster[ j++ ] = touchStatus[ i ] + 0x30;
            }
        msgFromMaster[ j++ ] = '\n';
        msgFromMaster[ j++ ] = 0;
        toMasterSerial.write( msgFromMaster );
        }
        break;
        default:
//             debug( "never come here\n" );
             toMasterSerial.write( msgFromMaster );
            break;
        }
    msgFromMaster[ 0 ] = 0;
    stateMaster = 0;
    }
}


//////////////////////////////////////////////////////////////
void loop( )
{
if ( bFirst )
    {
    ledOnOff( 12, 0 );
    bFirst = false;
    }

if ( bInit == true )
    {
    MPR121.updateAll( );
    for ( int i = 0; i < 12; i++ )
        {
        if ( MPR121.isNewTouch( i ) )
            {
            notifyKeyChanged( i, 1 );
            touchStatus[ i ] = 1;
            touchColors[ i ] = colorKey[ i ];
            }
        else if ( MPR121.isNewRelease( i ) )
            {
            notifyKeyChanged( i, 2 );
            touchStatus[ i ] = 0;
            touchColors[ i ] = 0;
            }
        }
    }
else
    {
    for ( int i = 0; i < 12; i++ )
        {
        touchColors[ i ] = 0;
        touchStatus[ i ] = 0;
        }
    }

tlcUpdate( );

if ( toMasterSerial.available( ) )
    {
    messageFromMaster( toMasterSerial.read( ) );
    }

if ( toSlaveSerial.available( ) )
    {
    messageFromSlave( toSlaveSerial.read( ) );
    }

}


